from flask import Flask, request, Response, g
from flask_httpauth import HTTPTokenAuth
from waitress import serve
from paste.translogger import TransLogger
import sys
import re
from os import listdir
from os.path import isfile, join, abspath, splitext, basename
from .datastore import MetricsDataStoreManager
from prometheus_client import CONTENT_TYPE_LATEST


class Exporter:
    def __init__(self, metrics_path, auth_token=None, host='0.0.0.0', port=5000):
        self.app = Flask(__name__)
        self.auth = HTTPTokenAuth(scheme='Bearer')
        self.metrics_path = metrics_path
        self.auth_token = auth_token
        self.host = host
        self.port = port

        @self.auth.verify_token
        def verify_token(token):
            # no auth
            if not self.auth_token:
                return True

            # check auth
            if token == self.auth_token:
                g.current_user = "prometheus"
                return True
            return False

        @self.app.route('/')
        @self.auth.login_required
        def metrics():
            data = {}
            filters = {}
            for param, value in request.args.items():
                if param.startswith('filter_'):
                    p = param.split("filter_")[1]
                    filters[p] = value

            key = request.args.get('key')

            for mf in self.get_metric_files():
                filename = basename(mf)
                name = splitext(filename)[0]
                manager = MetricsDataStoreManager.deserialize(
                    mf, filters=filters)
                data[name] = manager.dump_prometheus_metrics()

            # if we have a key filter only that 'type'
            # else convert everything to a response
            resp = b''.join(map(bytes, data.values()))

            if key:
                if key in data:
                    resp = data[key]
                else:
                    resp = b''

            return Response(resp, mimetype=CONTENT_TYPE_LATEST)

    def run(self):
        serve(TransLogger(self.app), host=self.host, port=self.port)

    def get_metric_files(self):
        metric_files = []
        if isfile(self.metrics_path):
            metric_files = [metrics_path]
        else:
            metric_files = [abspath(join(self.metrics_path, f)) for f in listdir(
                self.metrics_path) if isfile(join(self.metrics_path, f))]
        return metric_files
